
;    Copyright (c) 2007 Richard Ibbotson <richard.ibbotson@btinternet.com>
;
;    Permission is hereby granted, free of charge, to any person
;    obtaining a copy of this software and associated documentation
;    files (the "Software"), to deal in the Software without
;    restriction, including without limitation the rights to use, copy,
;    modify, merge, publish, distribute, sublicense, and/or sell copies
;    of the Software, and to permit persons to whom the Software is
;    furnished to do so, subject to the following conditions:
;
;    The above copyright notice and this permission notice shall be
;    included in all copies or substantial portions of the Software.
;
;    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
;    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
;    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
;    DEALINGS IN THE SOFTWARE.
;



;**********************************************************************
;                                                                     *
;    Filename:	    RoboBuilderPS2.asm                                           *
;    Date:             21st July 2008                               *
;    File Version:      V 0.1                                         *
;                                                                     *
;    Author:      Richard Ibbotson                                    *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    16F688.lkr                                       *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
; 
;
; 16F688 Chip Connections
;
; PORT		PIN		CONFIGURATION
; ==================================
; A0        13      PGM Header
; A1        12      PGM Header
; A2        11      PS2 ACK + 4.7K pullup to Vcc (not used in this code)
; C0        10      PS2 DAT + 4.7K pullup to Vcc
; C1        9       PS2 CMD 
; C2        8       PS2 CLK
; C3        7       PS2 ATT
; C4        6       Emulated IR Pulse output to RBC
; C5        5       N/C
; MCLR      4       PGM Header
; OSC2      3       N/C
; OSC1      2       N/C
; GND       14		GND
; VCC       1		Vcc
;

	list      p=16F688        ; list directive to define processor
	#include <p16F688.inc>    ; 
	errorlevel  -302          ; suppress message 302 from list file

; Here include Configuration settings
	__CONFIG   _CP_OFF & _CPD_OFF & _BOD_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _FCMEN_OFF & _IESO_OFF 

; OSC = INTOSCIO
; Watchdog = Disabled
; Power up Timer  = Enabled
; MCLR = Reset
; Brownout = Enabled, SBODEN disabled
; Switchover = Disabled
; Failsave = Disabled
; Code Protect Disable


;	Data Register Variables 	
	CBLOCK 0x20
; PS2 output buffer
	outlength		; number of bytes to send
	outbuf1			; start byte = 01			
	outbuf2	
	outbuf3
	outbuf4	
	outbuf5
	outbuf6	
	outbuf7
	outbuf8	
	outbuf9
	outbuf10
	outbuf11
	outbuf12	
	outbuf13
	outbuf14	
	outbuf15
	outbuf16	
	outbuf17
	outbuf18
	outbuf19	
	outbuf20
	outbuf21	
	outbuf22

; PS2 input buffer
	inbuf1			;Null					
	inbuf2			; Mode Number	
	inbuf3			; Mode changed
	inbuf4			; dpad switches
	inbuf5			; buttons
	inbuf6			; joystick right X
	inbuf7			; joystick right Y
	inbuf8			; joystick left X
	inbuf9			; joystick left Y
	inbuf10			; dpad right
	inbuf11			; dpad left
	inbuf12			; dpad up
	inbuf13			; dpad down
	inbuf14			; triangle
	inbuf15			; circle
	inbuf16			; cross
	inbuf17			; sqaure
	inbuf18			; Left 1
	inbuf19			; Right 1
	inbuf20			; Left 2
	inbuf21			; right 2
	inbuf22			; 



	outcount		; counter for the table read
	outshift		; output shift register
	PS2bytecount	; byte counter for PS2 transfer
	PS2bitcount		; bit counter for PS2 transfer
	IRbitcount		; bit count for IR transmit
	IRshift			; IR output shift register
	IRData			; IR Data to send
	msvar			; used for msdelay
	newcode			; code from PS2
	oldcode		; store of last code sent

	ENDC

	#define		BANK0	bcf		STATUS,RP0
	#define		BANK1	bsf		STATUS,RP0

;-----------------------------------------------------------------

	ORG 0x000				; Power on Reset
	nop
	clrf PCLATH
	goto start


	ORG 0x004				; Interrupt handler goes here

; Interrupts are not used
 
 	return					; return



; ---------- Main Program Starts Here ----------------------------

start
	
	call	chipinit		; Initialize pins, oscillator, etc
	call	IRinit			; initialise IR emulator
	call	PS2init			; initialise PS2 controller

	BANK0

mainloop
; delay 10mS
	call ms10delay

; read the PS2 controller
	call	readdata
	call	PS2inout
	

; encode buttons
checkstart
	btfsc	inbuf4, 3		; start pressed ?
	goto	checkselect
	call	encode2
	goto	encodedone
checkselect
	btfsc	inbuf4, 0		; select pressed ?
	goto	basicbutton
	call	encode1
	goto	encodedone
basicbutton

	call	encode0


; any button pressed ?
encodedone
	movwf	newcode ; save the code

	; same as previous ?
	subwf	oldcode,w
	btfsc	STATUS,Z
	goto	mainloop

	movf	newcode,w
	movwf	oldcode
	
	andlw	0ffh
	btfsc	STATUS,Z
	goto	mainloop

	movwf	IRData



; send the IRData
	call	IRSend			; send IR command

	goto	mainloop		; repeat forever

 
; ----------- Routines for Starting the Chip ---------

chipinit
	BANK0					; Memory bank 0
	movlw	01eh				; set output bits high
	movwf	PORTC
	movlw	07h				;
	movwf	CMCON0			; Comparator mode off

	BANK1					; Switch to memory bank 1
	movlw	21h				; Set the directions for port C
	movwf	TRISC			; 0,5 are in, 1,2,3,4 are out

	movlw	3ch				; Set the directions for port A
	movwf	TRISA			; 0,1 are out, 2,3,4,5 are in

	clrf	ANSEL			; Turn off A/D Converters

	bsf	OSCCON,6			; Set these three
	bsf	OSCCON,5			; bits to enable the
	bsf	OSCCON,4			; 8 MHz internal oscillator
	
	BANK0
	return


IRinit
	bsf		PORTC, 4		;Set output to high
	return

PS2init						; Set the PS2 controller to DS2 mode and lock
	call	ms10delay
	call	ms10delay
	call	configenter
	call	PS2inout
	call	ms1delay
	call	analoglock		; fill the output buffer
	call	PS2inout		; send the output buffer and get input buffer
	call	ms1delay
	call	DS2enter
	call	PS2inout
	call	DS2exit
	call	PS2inout
	call	configexit
	call	PS2inout
	call	readdata
	call	ms1delay
	call	PS2inout

	return


; ----------------- IR Remote Emulation routines

IRSend
; send preamble ( 3mS low, 1.8mS high)
	bcf		PORTC, 4		;set output to low
	call	ms3delay
	bsf		PORTC, 4		;set output to high
	call	ms1.8delay

; send Address
	movlw	02bh		; address for IR
	movwf	IRshift
	call	IROut
	
; send 0x02
	movlw	02h			; 02
	movwf	IRshift
	call	IROut

; send 0x00
	movlw	00h			; 00
	movwf	IRshift
	call	IROut

; send Data

	movf	IRData, w		; get data to send
	movwf	IRshift
	call	IROut
	
; send final pulse
	bcf		PORTC, 4		;set output to low
	call	ms0.6delay
	bsf		PORTC, 4		;set output to high
	return
;---------------------------------------------
; send 8 bits of data
IROut
	movlw	08h				; 8 bits
	movwf	IRbitcount	
IRloopbit
	bcf		PORTC, 4		;set output to low
	call	ms0.6delay
	bsf		PORTC, 4		;set output to high
	rrf		IRshift,f		; shift the output
	btfsc	STATUS, C		; is carry set
	call	ms0.5delay
	call	ms0.4delay
	decf	IRbitcount,f
	btfss	STATUS, Z
	goto	IRloopbit
	return
;----------------------------------------------------------------------------------------
;


readdata
	movlw	HIGH tblreaddata	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreaddata		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readloop
	movlw	HIGH tblreaddata	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreaddata		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readloop
;---------------------------------------------------------------------------------------
;
readshort
	movlw	HIGH tblreadshort	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreadshort		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readshortloop
	movlw	HIGH tblreadshort	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreadshort		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readshortloop
;---------------------------------------------------------------------------------------
;
readlong
	movlw	HIGH tblreadlong	;setup high table address
	movwf	PCLATH
	clrw
	movwf	outcount		; put it into table read counter
	call	tblreadlong		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
readlongloop
	movlw	HIGH tblreadlong	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblreadlong		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	readlongloop
;--------------------------------------------------------------------------------------
;
configenter
	movlw	HIGH tblconfigenter	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblconfigenter		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
centerloop
	movlw	HIGH tblconfigenter	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblconfigenter		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	centerloop
;---------------------------------------------------------------------------------------
;	
configexit
	movlw	HIGH tblconfigexit	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblconfigexit		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
cexitloop
	movlw	HIGH tblconfigexit	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblconfigexit		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	cexitloop
;--------------------------------------------------------------------------------------
;
DS2enter
	movlw	HIGH tblDS2native	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblDS2native		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
DS2entloop
	movlw	HIGH tblDS2native	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblDS2native		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	DS2entloop
;---------------------------------------------------------------------------------------
;
DS2exit
	movlw	HIGH tblDS2exit	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblDS2exit		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
DS2exitloop
	movlw	HIGH tblDS2exit	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblDS2exit		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	DS2exitloop	
;---------------------------------------------------------------------------------------
;
viben
	movlw	HIGH tblvibena	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblvibena		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
vibenloop
	movlw	HIGH tblvibena	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblvibena		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	vibenloop	
;---------------------------------------------------------------------------------------
;
vibdis
	movlw	HIGH tblvibdis	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblvibdis		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
vibdisloop
	movlw	HIGH tblvibdis	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblvibdis		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	vibdisloop	
;---------------------------------------------------------------------------------------
;
analoglock
	movlw	HIGH tblanalock	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tblanalock		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
analogloop
	movlw	HIGH tblanalock	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tblanalock		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	analogloop
;-------------------------------------------------------------------------------------
;

digitallock
	movlw	HIGH tbldiglock	;setup high table address
	movwf	PCLATH
	clrw					; offset is zero
	movwf	outcount		; put it into table read counter
	call	tbldiglock		; get the length
	movwf	outlength		; put into output length
	movlw	outbuf1			; fsr points to output buffer
	movwf	FSR
	incf	outcount,f		; increment the counter
digitalloop
	movlw	HIGH tbldiglock	;setup high table address
	movwf	PCLATH
	movf	outcount,w		; get the counter
	call	tbldiglock		; get the value
	movwf	INDF			; store it
	movf	outcount,w		; get the counter
	subwf	outlength,w		; are we done ?
	btfsc	STATUS, Z
	return					; yes all done	
	incf	FSR, f
	incf	outcount,f		; increment the counter
	goto	digitalloop

;-------------------------------------------------------------------------------------

; this is done this away to enable flexibility in assignment of codes to buttons
; change the order to change priority
; change the mapping if required
encode0
	btfss	inbuf5,4		;triangle		
	retlw	1h
	btfss	inbuf5,6		;cross
	retlw	2h
	btfss	inbuf5,2		;L1		
	retlw	3h
	btfss	inbuf4,4		;up
	retlw	4h
	btfss	inbuf5,3		;R1		
	retlw	5h
	btfss	inbuf4,7		;left
	retlw	6h
	btfss	inbuf5,7		;square		
	retlw	7h
	btfss	inbuf4,5		;right
	retlw	8h
	btfss	inbuf5,0		;L2		
	retlw	9h
	btfss	inbuf4,6		;down
	retlw	0ah
	btfss	inbuf5,1		;R2		
	retlw	0bh
	btfss	inbuf4,2		;joy right button
	retlw	0ch
	btfss	inbuf4,1		;joy left button
	retlw	0dh
	btfss	inbuf5,5		;circle
	retlw	16h
	
	movlw	32h
	subwf	inbuf6,w		;right joy left
	btfss	STATUS,C
	retlw	0eh

	movlw	32h
	addwf	inbuf7,w		;right joy up
	btfsc	STATUS,C
	retlw	11h

	movlw	32h				;right joy right
	addwf	inbuf6,w
	btfsc	STATUS,C
	retlw	10h

	movlw	32h
	subwf	inbuf7,w		;right joy down
	btfss	STATUS,C
	retlw	0fh

	movlw	32h				;left joy left
	subwf	inbuf8,w
	btfss	STATUS,C
	retlw	12h

	movlw	32h				;left joy up
	addwf	inbuf9,w
	btfsc	STATUS,C
	retlw	15h

	movlw	32h
	addwf	inbuf8,w		;left joy right
	btfsc	STATUS,C
	retlw	14h

	movlw	32h				;left joy down
	subwf	inbuf9,w
	btfss	STATUS,C
	retlw	13h

	retlw	00h

encode1
	btfss	inbuf5,6		; sel + cross		
	retlw	17h
	btfss	inbuf5,2		; sel + L1
	retlw	18h
	btfss	inbuf4,4		; sel + up		
	retlw	19h
	btfss	inbuf5,3		; sel + R1
	retlw	1ah
	btfss	inbuf4,7		; sel + left		
	retlw	1bh
	btfss	inbuf5,7		; sel + square
	retlw	1ch
	btfss	inbuf4,5		; sel + right		
	retlw	1dh
	btfss	inbuf5,0		; sel + L2
	retlw	1eh
	btfss	inbuf4,6		; sel + down		
	retlw	1fh
	btfss	inbuf5,1		; sel + R2
	retlw	20h
	btfss	inbuf4,2		; sel + joy right button				
	retlw	21h
	btfss	inbuf4,1		; sel + joy left button
	retlw	22h

	movlw	32h
	subwf	inbuf6,w		; sel + right joy left
	btfss	STATUS,C
	retlw	23h

	movlw	32h
	addwf	inbuf7,w		; sel + right joy up
	btfsc	STATUS,C
	retlw	26h

	movlw	32h
	addwf	inbuf6,w		;sel + right joy right
	btfsc	STATUS,C
	retlw	25h

	movlw	32h
	subwf	inbuf7,w		;sel + right joy down
	btfss	STATUS,C
	retlw	24h

	movlw	32h
	subwf	inbuf8,w		;sel + left joy left
	btfss	STATUS,C
	retlw	27h

	movlw	32h
	addwf	inbuf9,w		;sel + left joy up
	btfsc	STATUS,C
	retlw	2ah

	movlw	32h				;sel + left joy right
	addwf	inbuf8,w
	btfsc	STATUS,C
	retlw	29h

	movlw	32h
	subwf	inbuf9,w		;sel + left joy down
	btfss	STATUS,C
	retlw	28h

	retlw	00h

encode2
	btfss	inbuf5,4		;start + triangle		
	retlw	2bh
	btfss	inbuf5,6		;start + cross
	retlw	2ch
	btfss	inbuf5,2		;start + L1		
	retlw	2dh
	btfss	inbuf4,4		;start + forward
	retlw	2eh
	btfss	inbuf5,3		;start + R1		
	retlw	2fh
	btfss	inbuf4,7		;start + left
	retlw	30h
	btfss	inbuf5,7		;start + square		
	retlw	31h
	btfss	inbuf4,5		;start + right
	retlw	32h
	btfss	inbuf5,0		;start + L2		
	retlw	33h
	btfss	inbuf4,6		;start + down
	retlw	34h
	btfss	inbuf5,1		;start + R2		
	retlw	35h
	btfss	inbuf4,2		;start + right joy button
	retlw	36h
	btfss	inbuf4,1		;start + left joy button		
	retlw	37h
	
	movlw	32h				;start + right joy left
	subwf	inbuf6,w
	btfss	STATUS,C
	retlw	38h

	movlw	32h
	addwf	inbuf7,w		;start + right joy up
	btfsc	STATUS,C
	retlw	3bh

	movlw	32h
	addwf	inbuf6,w		;start + right joy right
	btfsc	STATUS,C
	retlw	3ah

	movlw	32h
	subwf	inbuf7,w		;start + right joy down
	btfss	STATUS,C
	retlw	39h

	movlw	32h
	subwf	inbuf8,w		;start + left joy left
	btfss	STATUS,C
	retlw	3ch

	movlw	32h				;start + left joy up
	addwf	inbuf9,w
	btfsc	STATUS,C
	retlw	3fh

	movlw	32h				;start + left joy right
	addwf	inbuf8,w
	btfsc	STATUS,C
	retlw	3eh

	movlw	32h				;start + left joy down
	subwf	inbuf9,w
	btfss	STATUS,C
	retlw	3dh

	retlw	00h
; end of encode


;----------------------------------------------------------------------------------
;Delays
; 8 Mhz oscillator 4 cycles per instruction = 500ns

ms1delay			; 1 ms delay
	clrf	msvar	; count for delay
delayloop1          ; (8 * 256) * .0005 = 1.024ms
	nop
	nop
	nop
	nop
	decf	msvar,f
	btfss	STATUS,Z
	goto delayloop1
	return
;-----------------------------------------
ms3delay
	call	ms1delay
	call	ms1delay
	call	ms1delay
	return

;-----------------------------------------

ms1.8delay
	call	ms1delay
	call	ms0.4delay
	call	ms0.4delay
	return
;-----------------------------------------

ms0.6delay
	movlw	96h
	movwf	msvar
delayloop6
	nop				;(8 * 150) * .005 = 0.6ms
	nop
	nop
	nop
	decf	msvar,f
	btfss	STATUS,Z
	goto delayloop6
	return
;--------------------------------------------

ms0.5delay
	clrf	msvar	; count for delay
delayloop5		; (4 * 256) * .0005 = 0.512ms

	decf	msvar,f
	btfss	STATUS,Z
	goto delayloop5
	return
;---------------------------------------------

ms0.4delay
	movlw	64h
	movwf	msvar
delayloop4
	nop				;(8 * 100) * .005 = 0.4ms
	nop
	nop
	nop
	decf	msvar,f
	btfss	STATUS,Z
	goto delayloop4
	return
;----------------------------------------------
ms10delay
	call	ms3delay
	call	ms3delay
	call	ms3delay
	call	ms1delay
	return

;-----------------------------------------------------------------------------------
; here we go to make the PS2 input/output
	
PS2inout
	clrf	PS2bytecount
	bcf		PORTC,3			; set ATT low
	
PS2loopbyte
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw	outbuf1			; fsr points to output buffer	
	movwf	FSR				
	movf	PS2bytecount, w ; get the byte counter
	addwf	FSR	,f			; add the offset
	movf	INDF,w			; get the value
	movwf	outshift		; put it into output shift reg

	movlw	inbuf1			; fsr points to input buffer	
	movwf	FSR				
	movf	PS2bytecount, w ; get the byte counter
	addwf	FSR, f				; add the offset
	movf	INDF,w			; get the value	


	movlw	08h				; eight bits
	movwf	PS2bitcount		; ready for first bit

PS2loopbit
	rrf		outshift,f		; shift the output
	btfsc	STATUS, C		; is carry set
	goto 	PS2outhi

	bcf		PORTC, 1		; set cmd bit low
	bcf		PORTC, 2		; set clk bit low
	goto	waitlow
PS2outhi
	bsf		PORTC, 1		; set cmd bit high
	bcf		PORTC, 2		; set clk bit low

waitlow
	rrf		INDF, f
	bsf		PORTC,2			; set clock high
	btfsc	PORTC,0			; get the data
	goto 	sethigh
	bcf		INDF, 7			; set the highest bit
	goto	indone
sethigh
	bsf		INDF, 7          ; clear the highest bit
indone

	decf	PS2bitcount,f
	btfss	STATUS, Z
	goto	PS2loopbit
	bsf		PORTC, 1		; remove data
	incf	PS2bytecount, f	; next byte count
	movf	outlength, w
	subwf	PS2bytecount,w
	btfss	STATUS, Z

	goto	PS2loopbyte
	bsf		PORTC, 3		; remove ATT
	return	
;-------------------------------------------------------------------------
;

	
; place the tables in seperate segment
	ORG 0x0500	

;-------------------------------------------------------------------------
;These are the PS2 command strings

tblreaddata
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 42h, 00h, 00h, 00h, 00h, 00h, 00h, 00h 

tblconfigenter
	addwf	PCL, f			; add the table offset
	dt		05h, 01h , 43h, 00h, 01h, 00h

tblconfigexit
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 43h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

tbldiglock
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 44h, 00h, 00h, 03h, 00h, 00h, 00h, 00h

tblanalock
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 44h, 00h, 01h, 03h, 00h, 00h, 00h, 00h 

tblvibena
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4dh, 00h, 00h, 01h, 0ffh, 0ffh, 0ffh, 0ffh 

tblvibdis
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4dh, 00h, 0ffh, 0ffh, 0ffh, 0ffh, 0ffh, 0ffh

tblDS2native
 	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 4fh, 00h, 0ffh, 0ffh, 03h, 00h, 00h, 00h

tblDS2exit

	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 43h, 00h, 00h, 5ah, 5ah, 5ah,5ah, 5ah

tblquerymode
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 45h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

tblqueryana
	addwf	PCL, f			; add the table offset
	dt		09h, 01h , 41h, 00h, 5ah, 5ah, 5ah, 5ah, 5ah, 5ah

tblreadshort
	addwf	PCL, f			; add the table offset
	dt		05h, 01h , 42h, 00h, 00h, 00h 

tblreadlong

	addwf	PCL, f			; add the table offset
	dt		16h, 01h , 42h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h  




	END


		 

